/* global Accept, wpforms_authorize_net */

'use strict';

/**
 * WPForms Authorize.Net function.
 *
 * @since 1.0.0
 */
var WPFormsAuthorizeNet = window.WPFormsAuthorizeNet || ( function( document, window, $ ) {

	/**
	 * Holder for original form submit handler.
	 *
	 * @since 1.4.0
	 *
	 * @type {Function}
	 */
	var originalSubmitHandler;

	/**
	 * Public functions and properties.
	 *
	 * @since 1.0.0
	 *
	 * @type {object}
	 */
	var app = {

		/**
		 * Start the engine.
		 *
		 * @since 1.0.0
		 */
		init: function() {

			$( document ).on( 'wpformsReady', app.updateSubmitHandler );

			// Document ready.
			$( app.ready );
		},

		/**
		 * Document ready.
		 *
		 * @since 1.1.0
		 */
		ready: function() {

			app.loadValidation();
		},

		/**
		 * Update submitHandler for forms containing Authorize.Net.
		 *
		 * @since 1.0.0
		 */
		updateSubmitHandler: function() {

			if ( typeof $.fn.validate === 'undefined' ) {
				return;
			}

			$( '.wpforms-authorize_net form' ).each( function() {

				var validator = $( this ).data( 'validator' );

				if ( ! validator ) {
					return true;
				}

				// Store the original submitHandler.
				originalSubmitHandler = validator.settings.submitHandler;

				// Replace the default submit handler.
				validator.settings.submitHandler = app.submitHandler;
			} );
		},

		/**
		 * Update submitHandler for forms containing Authorize.Net.
		 *
		 * @since 1.0.0
		 *
		 * @param {object} form JS form element.
		 */
		submitHandler: function( form ) {

			var $form = $( form );

			var cardData = {
				cardNumber: $form.find( '.wpforms-field-authorize_net-cardnumber' ).val().replace( /\s+/g, '' ),
				month     : $form.find( '.wpforms-field-authorize_net-cardmonth' ).val() || '',
				year      : $form.find( '.wpforms-field-authorize_net-cardyear' ).val() || '',
				cardCode  : $form.find( '.wpforms-field-authorize_net-cardcvc' ).val(),
			};

			var validator          = $form.data( 'validator' ),
				formValid          = ! ( validator && validator.numberOfInvalids() > 0 ),
				cardDataIncomplete = ! cardData.cardNumber || ! cardData.cardCode || ! cardData.year || ! cardData.month;

			// Omit sending card data to Access.js and call the original SubmitHandler
			// if the form is valid (not required) and card data is incomplete.
			if ( formValid && cardDataIncomplete ) {
				originalSubmitHandler( $form );
				return;
			}

			var authData = {
				clientKey : wpforms_authorize_net.public_client_key,
				apiLoginID: wpforms_authorize_net.api_login_id,
			};

			app.disableSubmitBtn( $form );

			Accept.dispatchData( { cardData: cardData, authData: authData }, function( response ) {
				app.apiResponseHandler( $form, response );
			} );
		},

		/**
		 * Handle Authorize.Net API response.
		 *
		 * @since 1.0.0
		 *
		 * @param {jQuery} $form Form element.
		 * @param {object} response API response.
		 */
		apiResponseHandler: function( $form, response ) {

			if ( response.messages.resultCode === 'Error' ) {
				app.displayApiErrorMessage( $form, response.messages.message );
				app.enableSubmitBtn( $form );
				return;
			}

			$form.append( '<input type="hidden" name="wpforms[authorize_net][opaque_data][descriptor]" value="' + app.escapeTextString( response.opaqueData.dataDescriptor ) + '">' );
			$form.append( '<input type="hidden" name="wpforms[authorize_net][opaque_data][value]" value="' + app.escapeTextString( response.opaqueData.dataValue ) + '">' );

			app.submitForm( $form );
		},

		/**
		 * Disable submit button for the form.
		 *
		 * @since 1.0.0
		 *
		 * @param {jQuery} $form Form element.
		 */
		disableSubmitBtn: function( $form ) {

			$form.find( '.wpforms-submit' ).prop( 'disabled', true );
		},

		/**
		 * Enable submit button for the form.
		 *
		 * @since 1.0.0
		 *
		 * @param {jQuery} $form Form element.
		 */
		enableSubmitBtn: function( $form ) {

			$form.find( '.wpforms-submit' ).prop( 'disabled', false );
		},

		/**
		 * Display Authorize.Net API error message for the form.
		 *
		 * @since 1.0.0
		 *
		 * @param {jQuery} $form Form element.
		 * @param {string} message Message to display.
		 */
		displayApiErrorMessage: function( $form, message ) {

			var $submitContainer = $form.find( '.wpforms-submit-container' ),
				$errorContainer = $( '<div class="wpforms-error-alert wpforms-error-authorize-net"></div>' ),
				errorText = '';

			for ( var i = 0; i < message.length; i++ ) {

				errorText = message[ i ].code + ': ' + message[ i ].text;
				$errorContainer.append( '<p>' + app.escapeTextString( errorText ) + '</p>' );
			}

			$submitContainer.siblings( '.wpforms-error-authorize-net' ).remove();
			$submitContainer.before( $errorContainer );
		},

		/**
		 * Submit the form using the original submitHandler.
		 *
		 * @since 1.0.0
		 *
		 * @param {jQuery} $form Form element.
		 */
		submitForm: function( $form ) {

			var validator = $form.data( 'validator' );

			if ( validator ) {
				originalSubmitHandler( $form );
			}
		},

		/**
		 * Load card validation.
		 *
		 * @since 1.0.0
		 */
		loadValidation: function() {

			// Credit card validation.
			if ( typeof $.fn.payment !== 'undefined' ) {
				$( '.wpforms-field-authorize_net-cardnumber' ).payment( 'formatCardNumber' );
				$( '.wpforms-field-authorize_net-cardcvc' ).payment( 'formatCardCVC' );
			}
		},

		/**
		 * Replaces &, <, >, ", `, and ' with their escaped counterparts.
		 *
		 * @since 1.0.0
		 *
		 * @param {string} string String to escape.
		 *
		 * @returns {string} Escaped string.
		 */
		escapeTextString: function( string ) {

			return $( '<span></span>' ).text( string ).html();
		},
	};

	// Provide access to public functions/properties.
	return app;

}( document, window, jQuery ) );

// Initialize.
WPFormsAuthorizeNet.init();
