<?php

namespace WPFormsAuthorizeNet;

use WPFormsAuthorizeNet\Migrations\Migrations;

/**
 * WPForms Authorize.Net loader class.
 *
 * @since 1.0.0
 */
final class Loader {

	/**
	 * Have the only available instance of the class.
	 *
	 * @since 1.0.0
	 *
	 * @var \WPFormsAuthorizeNet\Loader
	 */
	private static $instance;

	/**
	 * Authorize.Net credit card field.
	 *
	 * @since 1.0.0
	 *
	 * @var \WPFormsAuthorizeNet\Fields\AuthorizeNet
	 */
	public $field;

	/**
	 * Authorize.Net API.
	 *
	 * @since 1.0.0
	 *
	 * @var \WPFormsAuthorizeNet\Api\Api
	 */
	public $api;

	/**
	 * Magic getter.
	 *
	 * @since 1.1.0
	 *
	 * @param string $name Property name.
	 *
	 * @return string|null
	 */
	public function __get( $name ) {

		if ( $name === 'url' ) {
			_deprecated_argument( __CLASS__ . '->url', '1.1.0', 'Use WPFORMS_AUTHORIZE_NET_URL constant.' );

			return WPFORMS_AUTHORIZE_NET_URL;
		}

		return null;
	}

	/**
	 * Initiate main plugin instance.
	 *
	 * @since 1.0.0
	 *
	 * @return Loader
	 */
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {
			self::$instance = ( new Loader() )->init();
		}

		return self::$instance;
	}

	/**
	 * Init the Loader.
	 *
	 * @since 1.0.0
	 */
	public function init() {

		$this->hooks();

		return $this;
	}

	/**
	 * Plugin hooks.
	 *
	 * @since 1.0.0
	 */
	private function hooks() {

		add_action( 'wpforms_loaded', [ $this, 'setup' ], 15 );
	}

	/**
	 * All the actual plugin loading is done here.
	 *
	 * @since 1.0.0
	 */
	public function setup() {

		( new Migrations() )->init();

		$this->load_admin_entries();
		$this->load_settings();
		$this->load_field();
		$this->load_builder();
		$this->load_frontend();
		$this->load_api();
		$this->load_processing();
	}

	/**
	 * Load admin entries functionality.
	 *
	 * @since 1.0.0
	 */
	private function load_admin_entries() {

		if ( wpforms_is_admin_page( 'entries' ) ) {
			( new Admin\Entries() )->init();
		}
	}

	/**
	 * Load Authorize.Net settings.
	 *
	 * @since 1.0.0
	 */
	private function load_settings() {

		if ( wpforms_is_admin_page( 'settings', 'payments' ) ) {
			( new Admin\Settings() )->init();
		}
	}

	/**
	 * Load Authorize.Net field.
	 *
	 * @since 1.0.0
	 */
	private function load_field() {

		if ( ! is_admin() || wp_doing_ajax() || wpforms_is_admin_page( 'builder' ) ) {
			$this->field = new Fields\AuthorizeNet();
		}
	}

	/**
	 * Load builder functionality.
	 *
	 * @since 1.0.0
	 */
	private function load_builder() {

		if ( wpforms_is_admin_page( 'builder' ) ) {
			new Admin\AuthorizeNetPayment();
			( new Admin\Builder() )->init();
		}
	}

	/**
	 * Load frontend functionality.
	 *
	 * @since 1.0.0
	 */
	private function load_frontend() {

		if ( ! is_admin() ) {
			( new Frontend() )->init();
		}
	}

	/**
	 * Load Authorize.Net API.
	 *
	 * @since 1.0.0
	 */
	private function load_api() {

		if ( ! is_admin() || wpforms_is_frontend_ajax() || wpforms_is_admin_page( 'settings', 'payments' ) ) {
			$this->api = ( new Api\Api() )->init();
		}
	}

	/**
	 * Load payment form processing.
	 *
	 * @since 1.0.0
	 */
	private function load_processing() {

		if ( ! is_admin() || wpforms_is_frontend_ajax() ) {
			( new Process() )->init();
		}
	}

	/**
	 * Load the plugin updater.
	 *
	 * @since 1.0.0
	 * @deprecated 1.1.0
	 *
	 * @param string $key License key.
	 */
	public function updater( $key ) {

		_deprecated_function( __METHOD__, '1.1.0 of the WPForms Authorize.Net addon' );

		wpforms_authorize_net_updater( $key );
	}
}
