<?php

namespace WPFormsAuthorizeNet;

/**
 * Authorize.Net payment processing.
 *
 * @since 1.0.0
 */
class Process {

	/**
	 * Payment amount.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	public $amount = '';

	/**
	 * Form ID.
	 *
	 * @since 1.0.0
	 *
	 * @var int
	 */
	public $form_id = 0;

	/**
	 * Form Authorize.Net payment settings.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	public $settings = [];

	/**
	 * Sanitized submitted field values and data.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	public $fields = [];

	/**
	 * Form data and settings.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	public $form_data = [];

	/**
	 * Authorize.Net form errors.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	public $errors;

	/**
	 * Initialize.
	 *
	 * @since 1.0.0
	 */
	public function init() {

		$this->hooks();

		return $this;
	}

	/**
	 * Process hooks.
	 *
	 * @since 1.0.0
	 */
	public function hooks() {

		add_action( 'wpforms_process', [ $this, 'process_entry' ], 10, 4 );
		add_action( 'wpforms_authorize_net_api_set_rate_limit', [ $this, 'set_rate_limit' ] );
		add_action( 'wpforms_process_complete', [ $this, 'process_entry_meta' ], 10, 4 );
		add_filter( 'wpforms_entry_email_process', [ $this, 'process_email' ], 70, 4 );
	}

	/**
	 * Check if a payment exists with an entry, if so validate and process.
	 *
	 * @since 1.0.0
	 *
	 * @param array $fields    Final/sanitized submitted field data.
	 * @param array $entry     Copy of original $_POST.
	 * @param array $form_data Form data and settings.
	 */
	public function process_entry( $fields, $entry, $form_data ) {

		// Check if payment method exists and is enabled.
		if ( empty( $form_data['payments']['authorize_net']['enable'] ) ) {
			return;
		}

		$this->form_id   = (int) $form_data['id'];
		$this->fields    = $fields;
		$this->form_data = $form_data;
		$this->settings  = $form_data['payments']['authorize_net'];
		$this->amount    = wpforms_get_total_payment( $this->fields );

		// Before proceeding, check if any basic errors were detected.
		if ( ! $this->is_form_ok() ) {
			$this->display_errors();
			return;
		}

		// Set a card token provided by Accept.js.
		wpforms_authorize_net()->api->set_opaque_data( $entry );

		// Proceed to executing the purchase.
		$this->process_payment();

		// Update the card field value to contain basic details.
		$this->update_card_field_value();
	}

	/**
	 * Get form errors before payment processing.
	 *
	 * @since 1.0.0
	 */
	private function is_form_ok() {

		// Bail in case there are form processing errors.
		if ( ! empty( wpforms()->process->errors[ $this->form_id ] ) ) {
			return false;
		}

		if ( ! $this->is_card_field_visibility_ok() ) {
			return false;
		}

		// Check rate limit.
		if ( ! $this->is_rate_limit_ok() ) {
			$this->errors[] = esc_html__( 'Unable to process Authorize.Net payment, please try again later.', 'wpforms-authorize-net' );
			return false;
		}

		// Check for conditional logic.
		if ( ! $this->is_conditional_logic_ok( $this->settings ) ) {
			$title = esc_html__( 'Authorize.Net payment stopped by conditional logic.', 'wpforms-authorize-net' );
			$this->log_errors( $title, $this->fields, 'conditional_logic' );
			return false;
		}

		// Check for Authorize.Net keys.
		if ( ! Helpers::has_authorize_net_keys() ) {
			$this->errors[] = esc_html__( 'Authorize.Net payment stopped, missing API keys.', 'wpforms-authorize-net' );
			return false;
		}

		// Check total charge amount.
		// Authorize.Net has no minimum amount limit.
		if ( empty( $this->amount ) || wpforms_sanitize_amount( 0 ) === $this->amount ) {
			$this->errors[] = esc_html__( 'Authorize.Net payment stopped, invalid/empty amount.', 'wpforms-authorize-net' );
			return false;
		}

		// Check that, despite how the form is configured, the form and
		// entry actually contain payment fields, otherwise no need to proceed.
		if ( ! wpforms_has_payment( 'form', $this->form_data ) || ! wpforms_has_payment( 'entry', $this->fields ) ) {
			$this->errors[] = esc_html__( 'Authorize.Net payment stopped, missing payment fields.', 'wpforms-authorize-net' );
			return false;
		}

		// Check subscription settings are provided.
		if ( ! $this->is_subscription_settings_ok() ) {
			$this->errors[] = esc_html__( 'Authorize.Net payment stopped, missing required subscription settings.', 'wpforms-authorize-net' );
			return false;
		}

		return true;
	}

	/**
	 * Check if there is at least one visible (not hidden by conditional logic) card field in the form.
	 *
	 * @since 1.1.0
	 */
	private function is_card_field_visibility_ok() {

		// If the form contains no fields with conditional logic the card field is visible by default.
		if ( empty( $this->form_data['conditional_fields'] ) ) {
			return true;
		}

		foreach ( $this->fields as $field ) {

			if ( 'authorize_net' !== $field['type'] ) {
				continue;
			}

			// if the field is NOT in array of conditional fields, it's visible.
			if ( ! in_array( $field['id'], $this->form_data['conditional_fields'], true ) ) {
				return true;
			}

			// if the field IS in array of conditional fields and marked as visible, it's visible.
			if ( ! empty( $field['visible'] ) ) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Check if rate limit is under threshold and passes.
	 *
	 * @since 1.0.0
	 */
	private function is_rate_limit_ok() {

		$ip       = wpforms_get_ip();
		$ip_hash  = wp_hash( $ip );
		$attempts = get_transient( 'wpforms_authorize_net_attempt_' . $ip_hash );

		if ( ! empty( $attempts ) && $attempts >= 4 ) {
			$attempts++;
			set_transient( 'wpforms_authorize_net_attempt_' . $ip_hash, $attempts, HOUR_IN_SECONDS * 2 );
			return false;
		}

		return true;
	}

	/**
	 * Check if conditional logic check passes for the given settings.
	 *
	 * @since 1.0.0
	 *
	 * @param array $settings Conditional logic settings to process.
	 *
	 * @return bool
	 */
	private function is_conditional_logic_ok( $settings ) {

		// Check conditional logic settings.
		if (
			empty( $settings['conditional_logic'] ) ||
			empty( $settings['conditional_type'] ) ||
			empty( $settings['conditionals'] )
		) {
			return true;
		}

		// All conditional logic checks passed, continue with processing.
		$process = wpforms_conditional_logic()->process( $this->fields, $this->form_data, $settings['conditionals'] );

		if ( 'stop' === $settings['conditional_type'] ) {
			$process = ! $process;
		}

		return $process;
	}

	/**
	 * Check if subscription settings are provided.
	 *
	 * @since 1.0.0
	 *
	 * @return bool
	 */
	private function is_subscription_settings_ok() {

		if ( empty( $this->settings['recurring']['enable'] ) ) {
			return true;
		}

		if ( empty( $this->settings['recurring']['period'] ) ) {
			return false;
		}

		if ( empty( $this->settings['recurring']['email'] ) && $this->settings['recurring']['email'] !== '0' ) {
			return false;
		}

		if ( empty( $this->settings['recurring']['customer_name'] ) && $this->settings['recurring']['customer_name'] !== '0' ) {
			return false;
		}

		return true;
	}

	/**
	 * Process a payment.
	 *
	 * @since 1.0.0
	 */
	private function process_payment() {

		if ( empty( $this->settings['recurring']['enable'] ) ) {
			$this->process_payment_single();
		} else {
			$this->process_payment_subscription();
		}
	}

	/**
	 * Get args for any type of payment.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_payment_args_general() {

		return [
			'amount'   => $this->amount,
			'currency' => strtolower( wpforms_get_currency() ),
		];
	}

	/**
	 * Get single payment args.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_payment_args_single() {

		$args = $this->get_payment_args_general();

		// Customer first name.
		if ( ! empty( $this->fields[ $this->settings['customer_name'] ]['first'] ) ) {
			$args['customer_name']['first'] = sanitize_text_field( $this->fields[ $this->settings['customer_name'] ]['first'] );
		}

		// Customer last name.
		if ( ! empty( $this->fields[ $this->settings['customer_name'] ]['last'] ) ) {
			$args['customer_name']['last'] = sanitize_text_field( $this->fields[ $this->settings['customer_name'] ]['last'] );
		}

		// Receipt email.
		if ( ! empty( $this->fields[ $this->settings['receipt_email'] ]['value'] ) ) {
			$args['receipt_email'] = sanitize_email( $this->fields[ $this->settings['receipt_email'] ]['value'] );
		}

		// Payment description.
		if ( ! empty( $this->settings['payment_description'] ) ) {
			$args['description'] = html_entity_decode( $this->settings['payment_description'], ENT_COMPAT, 'UTF-8' );
		}

		// Customer billing address.
		if ( ! empty( $this->fields[ $this->settings['customer_billing_address'] ] ) ) {
			$args['customer_billing_address'] = $this->fields[ $this->settings['customer_billing_address'] ];
		}

		return $args;
	}

	/**
	 * Get subscription payment args.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_payment_args_subscription() {

		$args     = $this->get_payment_args_general();
		$args_sub = [];

		// Receipt email.
		if ( ! empty( $this->fields[ $this->settings['recurring']['email'] ]['value'] ) ) {
			$args_sub['email'] = sanitize_email( $this->fields[ $this->settings['recurring']['email'] ]['value'] );
		}

		// Customer first name.
		if ( ! empty( $this->fields[ $this->settings['recurring']['customer_name'] ]['first'] ) ) {
			$args_sub['customer_name']['first'] = sanitize_text_field( $this->fields[ $this->settings['recurring']['customer_name'] ]['first'] );
		}

		// Customer last name.
		if ( ! empty( $this->fields[ $this->settings['recurring']['customer_name'] ]['last'] ) ) {
			$args_sub['customer_name']['last'] = sanitize_text_field( $this->fields[ $this->settings['recurring']['customer_name'] ]['last'] );
		}

		$period_data = Helpers::get_subscription_period_data();

		// Subscription period.
		$args_sub['period'] = array_key_exists( $this->settings['recurring']['period'], $period_data ) ? $period_data[ $this->settings['recurring']['period'] ] : $period_data['yearly'];

		// Subscription start date.
		$args_sub['start_date'] = date_create( '+' . $args_sub['period']['count'] . $args_sub['period']['unit'] );

		$title = ! empty( $this->settings['recurring']['name'] ) ? $this->settings['recurring']['name'] : sanitize_text_field( $this->form_data['settings']['form_title'] );

		// Subscription name.
		$args_sub['name'] = sprintf( '%s (%s %s)', $title, $args['amount'], $args_sub['period']['desc'] );

		// Customer billing address.
		if ( ! empty( $this->fields[ $this->settings['recurring']['customer_billing_address'] ] ) ) {
			$args_sub['customer_billing_address'] = $this->fields[ $this->settings['recurring']['customer_billing_address'] ];
		}

		// Subscription regular payment description.
		/* translators: %s - Name of the subscription plan. */
		$args_sub['payment_desc'] = sprintf( __( 'Payment for %s', 'wpforms-authorize-net' ), esc_html( $title ) );

		// Subscription settlement payment description.
		/* translators: %s - Name of the subscription plan. */
		$args_sub['settlement_desc'] = sprintf( __( 'Settlement Payment for %s', 'wpforms-authorize-net' ), esc_html( $title ) );

		// Put subscription arguments into its own key.
		$args['subscription'] = $args_sub;

		return $args;
	}

	/**
	 * Process a single payment.
	 *
	 * @since 1.0.0
	 */
	private function process_payment_single() {

		$args = $this->get_payment_args_single();
		$args = apply_filters( 'wpforms_authorize_net_process_payment_single_args', $args, $this );

		wpforms_authorize_net()->api->process_transaction( $args );

		$this->process_api_errors( 'single' );
	}

	/**
	 * Process a subscription payment.
	 *
	 * @since 1.0.0
	 */
	private function process_payment_subscription() {

		// Check for conditional logic.
		if ( ! $this->is_conditional_logic_ok( $this->settings['recurring'] ) ) {
			$this->process_payment_single();
			return;
		}

		$args = $this->get_payment_args_subscription();
		$args = apply_filters( 'wpforms_authorize_net_process_payment_subscription_args', $args, $this );

		wpforms_authorize_net()->api->process_subscription( $args );

		$this->process_api_errors( 'subscription' );
	}

	/**
	 * Display form errors.
	 *
	 * @since 1.0.0
	 *
	 * @param array $errors Errors to display.
	 */
	private function display_errors( $errors = [] ) {

		if ( ! $errors ) {
			$errors = $this->errors;
		}

		if ( ! $errors || ! is_array( $errors ) ) {
			return;
		}

		// Check if the form contains a required credit card. If it does
		// and there was an error, return the error to the user and prevent
		// the form from being submitted. This should not occur under normal
		// circumstances.
		foreach ( $this->form_data['fields'] as $field ) {

			if ( 'authorize_net' !== $field['type'] ) {
				continue;
			}

			if ( ! empty( $field['required'] ) ) {
				wpforms()->process->errors[ $this->form_id ]['footer'] = implode( '<br>', $errors );
			}
		}
	}

	/**
	 * Collect errors from API and turn it into form errors.
	 *
	 * @since 1.0.0
	 *
	 * @param string $type Payment time (e.g. 'single' or 'subscription').
	 */
	private function process_api_errors( $type ) {

		$errors = wpforms_authorize_net()->api->get_errors();

		if ( empty( $errors ) || ! is_array( $errors ) ) {
			return;
		}

		$this->display_errors( $errors );

		if ( 'subscription' === $type ) {
			$title = esc_html__( 'Authorize.Net subscription payment stopped by error', 'wpforms-authorize-net' );
		} else {
			$title = esc_html__( 'Authorize.Net payment stopped by error', 'wpforms-authorize-net' );
		}

		// Log transaction specific errors to make payment issues identification easier.
		$_errors = wpforms_authorize_net()->api->response->extract_errors_from_transaction_part_raw();

		if ( $_errors ) {
			$errors[] = $_errors;
		}

		$this->log_errors( $title, $errors );
	}

	/**
	 * Set rate limit for API errors.
	 *
	 * @since 1.0.0
	 *
	 * @param mixed $response Authorize.Net response.
	 */
	public function set_rate_limit( $response ) {

		if ( Helpers::is_test_mode() ) {
			return;
		}

		if ( ! apply_filters( 'wpforms_authorize_net_process_set_rate_limit', true ) ) {
			return;
		}

		$ip       = wpforms_get_ip();
		$ip_hash  = wp_hash( $ip );
		$attempts = get_transient( 'wpforms_authorize_net_attempt_' . $ip_hash );

		if ( false === $attempts ) {
			$attempts = 1;
		} else {
			$attempts++;
		}

		set_transient( 'wpforms_authorize_net_attempt_' . $ip_hash, $attempts, HOUR_IN_SECONDS * 2 );
	}

	/**
	 * Log payment errors.
	 *
	 * @since 1.0.0
	 *
	 * @param string       $title    Error title.
	 * @param array|string $messages Error messages.
	 * @param string       $level    Error level to add to 'payment' error level.
	 */
	private function log_errors( $title, $messages = [], $level = 'error' ) {

		wpforms_log(
			$title,
			$messages,
			[
				'type'    => [ 'payment', $level ],
				'form_id' => $this->form_id,
			]
		);
	}

	/**
	 * Update the card field value to contain basic details.
	 *
	 * @since 1.0.0
	 */
	private function update_card_field_value() {

		if ( $this->errors ) {
			return;
		}

		foreach ( $this->fields as $field_id => $field ) {

			if ( $field['type'] !== 'authorize_net' ) {
				continue;
			}

			$details = [
				'brand' => wpforms_authorize_net()->api->response->get_transaction_card_brand(),
				'last4' => wpforms_authorize_net()->api->response->get_transaction_card_last4(),
			];

			$details = implode( "\n", array_filter( $details ) );
			$details = apply_filters(
				'wpforms_authorize_net_process_update_card_field_value',
				$details,
				wpforms_authorize_net()->api->response->get_transaction_response()
			);

			wpforms()->process->fields[ $field_id ]['value'] = $details;

			return;
		}
	}

	/**
	 * Update entry details and add meta for a successful payment.
	 *
	 * @since 1.0.0
	 *
	 * @param array  $fields    Final/sanitized submitted field data.
	 * @param array  $entry     Copy of original $_POST.
	 * @param array  $form_data Form data and settings.
	 * @param string $entry_id  Entry ID.
	 */
	public function process_entry_meta( $fields, $entry, $form_data, $entry_id ) {

		if ( $this->errors || empty( $entry_id ) ) {
			return;
		}

		$transaction_id = wpforms_authorize_net()->api->response->get_transaction_id();

		if ( empty( $transaction_id ) ) {
			return;
		}

		$subscription_id = wpforms_authorize_net()->api->response->get_subscription_id();
		$customer_id     = wpforms_authorize_net()->api->response->get_subscription_profile_id();

		$meta = [
			'payment_type'         => 'authorize_net',
			'payment_total'        => $this->amount,
			'payment_currency'     => wpforms_get_currency(),
			'payment_transaction'  => sanitize_text_field( $transaction_id ),
			'payment_mode'         => 'live' === Helpers::get_authorize_net_mode() ? 'production' : 'test',
			'payment_subscription' => ! empty( $subscription_id ) ? sanitize_text_field( $subscription_id ) : '',
			'payment_customer'     => ! empty( $customer_id ) ? sanitize_text_field( $customer_id ) : '',
			'payment_period'       => ! empty( $subscription_id ) ? sanitize_text_field( $this->settings['recurring']['period'] ) : '',
		];

		$data = [
			'status' => 'completed',
			'type'   => 'payment',
			'meta'   => wp_json_encode( $meta ),
		];

		wpforms()->entry->update( $entry_id, $data, '', '', [ 'cap' => false ] );

		// Insert payment data into entry_meta table.
		wpforms()->get( 'entry' )->insert_payment_meta( $entry_id, $meta );

		$transaction  = wpforms_authorize_net()->api->response->get_transaction_response();
		$subscription = wpforms_authorize_net()->api->response->get_subscription_response();
		$customer     = wpforms_authorize_net()->api->response->get_subscription_profile_part();

		// Processing complete.
		do_action( 'wpforms_authorize_net_process_entry_meta', $fields, $form_data, $entry_id, $transaction, $subscription, $customer );
	}

	/**
	 * Logic that helps decide if we should send completed payments notifications.
	 *
	 * @since 1.1.0
	 *
	 * @param bool  $process         Whether to process or not.
	 * @param array $fields          Form fields.
	 * @param array $form_data       Form data.
	 * @param int   $notification_id Notification ID.
	 *
	 * @return bool
	 */
	public function process_email( $process, $fields, $form_data, $notification_id ) {

		if ( ! $process ) {
			return false;
		}

		if ( empty( $form_data['payments']['authorize_net']['enable'] ) ) {
			return $process;
		}

		if ( empty( $form_data['settings']['notifications'][ $notification_id ]['authorize_net'] ) ) {
			return $process;
		}

		if ( ! $this->is_conditional_logic_ok( $this->settings ) ) {
			return false;
		}

		if ( ! wpforms_authorize_net()->api->response->is_whole_transaction_ok() && ! wpforms_authorize_net()->api->response->is_whole_subscription_ok() ) {
			return false;
		}

		return empty( $this->errors ) && empty( wpforms_authorize_net()->api->get_errors() );
	}
}
