<?php
/**
 * Plugin Name:       WPForms Authorize.Net
 * Plugin URI:        https://wpforms.com
 * Description:       Authorize.Net integration with WPForms.
 * Requires at least: 5.2
 * Requires PHP:      5.6
 * Author:            WPForms
 * Author URI:        https://wpforms.com
 * Version:           1.4.0
 * Text Domain:       wpforms-authorize-net
 * Domain Path:       languages
 *
 * WPForms is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * any later version.
 *
 * WPForms is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WPForms. If not, see <https://www.gnu.org/licenses/>.
 */

use WPFormsAuthorizeNet\Loader;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Plugin constants.
 *
 * @since 1.0.0
 */
// phpcs:disable WPForms.Comments.PHPDocDefine.MissPHPDoc
define( 'WPFORMS_AUTHORIZE_NET_VERSION', '1.4.0' );
define( 'WPFORMS_AUTHORIZE_NET_FILE', __FILE__ );
define( 'WPFORMS_AUTHORIZE_NET_PATH', plugin_dir_path( WPFORMS_AUTHORIZE_NET_FILE ) );
define( 'WPFORMS_AUTHORIZE_NET_URL', plugin_dir_url( WPFORMS_AUTHORIZE_NET_FILE ) );
define( 'WPFORMS_AUTHORIZE_NET_MIN_WPFORMS_VERSION', '1.7.5.5' );
// phpcs:enable WPForms.Comments.PHPDocDefine.MissPHPDoc

/**
 * Load the provider class.
 *
 * @since 1.1.0
 */
function wpforms_authorize_net_load() {

	// Check requirements.
	if ( ! wpforms_authorize_net_required() ) {
		return;
	}

	// Load the plugin.
	wpforms_authorize_net();
}

add_action( 'wpforms_loaded', 'wpforms_authorize_net_load' );

/**
 * Check addon requirements.
 *
 * @since 1.1.0
 */
function wpforms_authorize_net_required() {

	if ( PHP_VERSION_ID < 50600 ) {
		add_action( 'admin_init', 'wpforms_authorize_net_deactivate' );
		add_action( 'admin_notices', 'wpforms_authorize_net_fail_php_version' );

		return false;
	}

	if ( ! function_exists( 'wpforms' ) ) {
		return false;
	}

	if ( version_compare( wpforms()->version, WPFORMS_AUTHORIZE_NET_MIN_WPFORMS_VERSION, '<' ) ) {
		add_action( 'admin_init', 'wpforms_authorize_net_deactivate' );
		add_action( 'admin_notices', 'wpforms_authorize_net_fail_wpforms_version' );

		return false;
	}

	if ( ! function_exists( 'wpforms_get_license_type' ) || ! in_array( wpforms_get_license_type(), [ 'elite', 'agency', 'ultimate' ], true ) ) {
		return false;
	}

	return true;
}

/**
 * Deactivate the plugin.
 *
 * @since 1.0.0
 */
function wpforms_authorize_net_deactivate() {

	deactivate_plugins( plugin_basename( WPFORMS_AUTHORIZE_NET_FILE ) );
}

/**
 * Admin notice for a minimum PHP version.
 *
 * @since 1.1.0
 */
function wpforms_authorize_net_fail_php_version() {

	echo '<div class="notice notice-error"><p>';
	printf(
		wp_kses( /* translators: %s - WPForms.com documentation page URL. */
			__( 'The WPForms Authorize.Net plugin has been deactivated. Your site is running an outdated version of PHP that is no longer supported and is not compatible with the Authorize.Net addon. <a href="%s" target="_blank" rel="noopener noreferrer">Read more</a> for additional information.', 'wpforms-authorize-net' ),
			[
				'a' => [
					'href'   => [],
					'rel'    => [],
					'target' => [],
				],
			]
		),
		'https://wpforms.com/docs/supported-php-version/'
	);
	echo '</p></div>';

	// phpcs:disable WordPress.Security.NonceVerification.Recommended
	if ( isset( $_GET['activate'] ) ) {
		unset( $_GET['activate'] );
	}
	// phpcs:enable WordPress.Security.NonceVerification.Recommended
}

/**
 * Display notice after deactivation.
 *
 * @since 1.0.0
 * @deprecated 1.1.0
 */
function wpforms_authorize_net_deactivate_msg() {

	_deprecated_function( __FUNCTION__, '1.1.0 of the WPForms Authorize.Net addon' );

	wpforms_authorize_net_fail_wpforms_version();
}

/**
 * Admin notice for minimum WPForms version.
 *
 * @since 1.1.0
 */
function wpforms_authorize_net_fail_wpforms_version() {

	echo '<div class="notice notice-error"><p>';
	// phpcs:disable WordPress.Security.EscapeOutput.OutputNotEscaped
	printf(
		/* translators: Minimum required WPForms version. */
		esc_html__( 'The WPForms Authorize.Net plugin has been deactivated, because it requires WPForms v%s or later to work.', 'wpforms-authorize-net' ),
		WPFORMS_AUTHORIZE_NET_MIN_WPFORMS_VERSION
	);
	// phpcs:enable WordPress.Security.EscapeOutput.OutputNotEscaped
	echo '</p></div>';

	// phpcs:disable WordPress.Security.NonceVerification.Recommended
	if ( isset( $_GET['activate'] ) ) {
		unset( $_GET['activate'] );
	}
	// phpcs:enable WordPress.Security.NonceVerification.Recommended
}

/**
 * Get the instance of the plugin main class, which actually loads all the code.
 *
 * @since 1.0.0
 *
 * @return Loader
 */
function wpforms_authorize_net() {

	require_once WPFORMS_AUTHORIZE_NET_PATH . 'vendor/autoload.php';

	return Loader::get_instance();
}

/**
 * Load the plugin updater.
 *
 * @since 1.1.0
 *
 * @param string $key License key.
 */
function wpforms_authorize_net_updater( $key ) {

	new WPForms_Updater(
		[
			'plugin_name' => 'WPForms Authorize.Net',
			'plugin_slug' => 'wpforms-authorize-net',
			'plugin_path' => plugin_basename( WPFORMS_AUTHORIZE_NET_FILE ),
			'plugin_url'  => trailingslashit( WPFORMS_AUTHORIZE_NET_URL ),
			'remote_url'  => WPFORMS_UPDATER_API,
			'version'     => WPFORMS_AUTHORIZE_NET_VERSION,
			'key'         => $key,
		]
	);
}

add_action( 'wpforms_updater', 'wpforms_authorize_net_updater' );
