<?php

/**
 * PayPal Standard integration.
 *
 * @since 1.0.0
 */
class WPForms_Paypal_Standard extends WPForms_Payment {

	/**
	 * Production mode.
	 *
	 * @since 1.5.0
	 *
	 * @var string
	 */
	const PRODUCTION = 'production';

	/**
	 * Initialize.
	 *
	 * @since 1.0.0
	 */
	public function init() {

		$this->version  = WPFORMS_PAYPAL_STANDARD_VERSION;
		$this->name     = 'PayPal Standard';
		$this->slug     = 'paypal_standard';
		$this->priority = 10;
		$this->icon     = plugins_url( 'assets/images/addon-icon-paypal.png', __FILE__ );

		add_action( 'wpforms_process_complete', [ $this, 'process_entry' ], 20, 4 );
		add_action( 'init', [ $this, 'process_ipn' ] );
		add_action( 'wpforms_form_settings_notifications_single_after', [ $this, 'notification_settings' ], 10, 2 );
		add_filter( 'wpforms_entry_email_process', [ $this, 'process_email' ], 70, 5 );
		add_action( 'wpforms_builder_enqueues', [ $this, 'enqueues' ] );
		add_filter( 'wpforms_builder_strings', [ $this, 'javascript_strings' ], 10, 2 );
	}

	/**
	 * Display content inside the panel content area.
	 *
	 * @since 1.0.0
	 */
	public function builder_content() {

		wpforms_panel_field(
			'toggle',
			$this->slug,
			'enable',
			$this->form_data,
			esc_html__( 'Enable PayPal Standard payments', 'wpforms-paypal-standard' ),
			[
				'parent'  => 'payments',
				'default' => '0',
			]
		);

		echo '<div class="wpforms-panel-content-section-paypal-standard-body">';

		$this->builder_email_field();

		wpforms_panel_field(
			'select',
			$this->slug,
			'mode',
			$this->form_data,
			esc_html__( 'Mode', 'wpforms-paypal-standard' ),
			[
				'parent'  => 'payments',
				'default' => self::PRODUCTION,
				'options' => [
					'production' => esc_html__( 'Production', 'wpforms-paypal-standard' ),
					'test'       => esc_html__( 'Test / Sandbox', 'wpforms-paypal-standard' ),
				],
				'tooltip' => esc_html__( 'Select Production to receive real payments or select Test to use the PayPal developer sandbox', 'wpforms-paypal-standard' ),
			]
		);

		wpforms_panel_field(
			'select',
			$this->slug,
			'transaction',
			$this->form_data,
			esc_html__( 'Payment Type', 'wpforms-paypal-standard' ),
			[
				'parent'  => 'payments',
				'default' => 'product',
				'options' => [
					'product'  => esc_html__( 'Products and Services', 'wpforms-paypal-standard' ),
					'donation' => esc_html__( 'Donation', 'wpforms-paypal-standard' ),
				],
				'tooltip' => esc_html__( 'Select the type of payment you are receiving.', 'wpforms-paypal-standard' ),
			]
		);

		wpforms_panel_field(
			'text',
			$this->slug,
			'cancel_url',
			$this->form_data,
			esc_html__( 'Cancel URL', 'wpforms-paypal-standard' ),
			[
				'parent'  => 'payments',
				'tooltip' => esc_html__( 'Enter the URL to send users to if they do not complete the PayPal checkout', 'wpforms-paypal-standard' ),
			]
		);

		wpforms_panel_field(
			'select',
			$this->slug,
			'shipping',
			$this->form_data,
			esc_html__( 'Shipping', 'wpforms-paypal-standard' ),
			[
				'parent'  => 'payments',
				'default' => '0',
				'options' => [
					'1' => esc_html__( 'Don\'t ask for an address', 'wpforms-paypal-standard' ),
					'0' => esc_html__( 'Ask for an address, but do not require', 'wpforms-paypal-standard' ),
					'2' => esc_html__( 'Ask for an address and require it', 'wpforms-paypal-standard' ),
				],
			]
		);

		wpforms_conditional_logic()->builder_block(
			[
				'form'        => $this->form_data,
				'type'        => 'panel',
				'panel'       => $this->slug,
				'parent'      => 'payments',
				'actions'     => [
					'go'   => esc_html__( 'Process', 'wpforms-paypal-standard' ),
					'stop' => esc_html__( 'Don\'t process', 'wpforms-paypal-standard' ),
				],
				'action_desc' => esc_html__( 'this charge if', 'wpforms-paypal-standard' ),
				'reference'   => esc_html__( 'PayPal Standard payment', 'wpforms-paypal-standard' ),
			]
		);

		echo '</div>';
	}

	/**
	 * Add email field.
	 *
	 * @since 1.5.0
	 */
	private function builder_email_field() {

		$is_production_mode = empty( $this->form_data['payments'][ $this->slug ] ) || $this->is_production_mode();

		// Backward compatibility for forms created on addon's version < 1.5.0.
		$fallback_email = ! empty( $this->form_data['payments'][ $this->slug ]['email'] ) ? $this->form_data['payments'][ $this->slug ]['email'] : '';

		wpforms_panel_field(
			'text',
			$this->slug,
			'production_email',
			$this->form_data,
			esc_html__( 'PayPal Email Address (Production)', 'wpforms-paypal-standard' ),
			[
				'parent'        => 'payments',
				'placeholder'   => esc_html__( 'Enter your business email address', 'wpforms-paypal-standard' ),
				'class'         => ! $is_production_mode ? 'wpforms-hidden' : '',
				'default'       => ! $is_production_mode ? '' : $fallback_email,
				'after_tooltip' => '&nbsp;<span class="required">*</span>',
				'input_class'   => 'wpforms-required',
			]
		);

		wpforms_panel_field(
			'text',
			$this->slug,
			'sandbox_email',
			$this->form_data,
			esc_html__( 'PayPal Email Address (Sandbox)', 'wpforms-paypal-standard' ),
			[
				'parent'        => 'payments',
				'placeholder'   => esc_html__( 'Enter your business email address', 'wpforms-paypal-standard' ),
				'class'         => $is_production_mode ? 'wpforms-hidden' : '',
				'default'       => $is_production_mode ? '' : $fallback_email,
				'after_tooltip' => '&nbsp;<span class="required">*</span>',
				'input_class'   => 'wpforms-required',
			]
		);
	}

	/**
	 * Process and submit entry to provider.
	 *
	 * @since 1.0.0
	 *
	 * @param array $fields
	 * @param array $entry
	 * @param array $form_data
	 * @param int   $entry_id
	 */
	public function process_entry( $fields, $entry, $form_data, $entry_id ) {

		$this->form_data = $form_data;
		$errors          = [];

		// Check an entry was created and passed.
		if ( empty( $entry_id ) ) {
			return;
		}

		// Check if payment method exists.
		if ( empty( $this->form_data['payments'][ $this->slug ] ) ) {
			return;
		}

		// Check required payment settings.
		$payment_settings = $this->form_data['payments'][ $this->slug ];
		$email            = $this->get_email();

		if (
			empty( $email ) ||
			empty( $payment_settings['enable'] ) ||
			$payment_settings['enable'] != '1'
		) {
			return;
		}

		if ( ! empty( wpforms()->get( 'process' )->errors[ $this->form_data['id'] ] ) ) {
			return;
		}

		// If preventing the notification, log it, and then bail.
		if ( ! $this->is_conditional_logic_ok( $fields ) ) {
			$this->log_errors( esc_html__( 'PayPal Standard Payment stopped by conditional logic', 'wpforms-paypal-standard' ), $fields, 'conditional_logic' );

			return;
		}

		// Check that, despite how the form is configured, the form and
		// entry actually contain payment fields, otherwise no need to proceed.
		$form_has_payments  = wpforms_has_payment( 'form', $this->form_data );
		$entry_has_paymemts = wpforms_has_payment( 'entry', $fields );

		if ( ! $form_has_payments || ! $entry_has_paymemts ) {
			$error_title = esc_html__( 'PayPal Standard Payment stopped, missing payment fields', 'wpforms-paypal-standard' );
			$errors[]    = $error_title;

			$this->log_errors( $error_title );
		}

		// Check total charge amount.
		$amount = wpforms_get_total_payment( $fields );

		if ( empty( $amount ) || $amount === wpforms_sanitize_amount( 0 ) ) {
			$error_title = esc_html__( 'PayPal Standard Payment stopped, invalid/empty amount', 'wpforms-paypal-standard' );
			$errors[]    = $error_title;

			$this->log_errors( $error_title );
		}

		if ( $errors ) {

			$this->display_errors( $errors );

			return;
		}

		// Update entry to include payment details.
		$payment_data = [
			'payment_type'      => $this->slug,
			'payment_recipient' => trim( sanitize_email( $email ) ),
			'payment_total'     => $amount,
			'payment_currency'  => strtolower( wpforms_get_currency() ),
			'payment_mode'      => esc_html( $payment_settings['mode'] ),
		];

		$entry_data = array(
			'status' => 'pending',
			'type'   => 'payment',
			'meta'   => wp_json_encode( $payment_data ),
		);
		wpforms()->entry->update( $entry_id, $entry_data, '', '', array( 'cap' => false ) );

		// Insert payment data into entry_meta table.
		wpforms()->get( 'entry' )->insert_payment_meta( $entry_id, $payment_data );

		// Build the return URL with hash.
		$query_args  = 'form_id=' . $this->form_data['id'] . '&entry_id=' . $entry_id . '&hash=' . wp_hash( $this->form_data['id'] . ',' . $entry_id );
		$return_url  = is_ssl() ? 'https://' : 'http://';
		$return_url .= $_SERVER['SERVER_NAME'] . $_SERVER['REQUEST_URI'];
		if ( ! empty( $this->form_data['settings']['ajax_submit'] ) && ! empty( $_SERVER['HTTP_REFERER'] ) ) {
			$return_url = $_SERVER['HTTP_REFERER'];
		}

		$return_url = esc_url_raw(
			add_query_arg(
				[
					'wpforms_return' => base64_encode( $query_args ),
				],
				/**
				 * PayPal Standard return URL.
				 *
				 * @since 1.0.0
				 *
				 * @param string $return_url Return URL.
				 * @param array  $form_data  Form data.
				 */
				apply_filters( 'wpforms_paypal_return_url', $return_url, $this->form_data ) // phpcs:ignore WPForms.PHP.ValidateHooks.InvalidHookName
			)
		);

		// Setup various vars.
		$items       = wpforms_get_payment_items( $fields );
		$redirect    = $this->is_production_mode() ? 'https://www.paypal.com/cgi-bin/webscr/?' : 'https://www.sandbox.paypal.com/cgi-bin/webscr/?';
		$cancel_url  = ! empty( $payment_settings['cancel_url'] ) ? esc_url_raw( $payment_settings['cancel_url'] ) : home_url();
		$transaction = 'donation' === $payment_settings['transaction'] ? '_donations' : '_cart';

		// Setup PayPal arguments.
		$paypal_args = array(
			'bn'            => 'WPForms_SP',
			'lc'            => get_user_locale(),
			'business'      => trim( $email ),
			'cancel_return' => $cancel_url,
			'cbt'           => get_bloginfo( 'name' ),
			'charset'       => get_bloginfo( 'charset' ),
			'cmd'           => $transaction,
			'currency_code' => strtoupper( wpforms_get_currency() ),
			'custom'        => absint( $this->form_data['id'] ),
			'invoice'       => sprintf( '%s-%d', hash( 'crc32b', home_url() ), absint( $entry_id ) ),
			'no_shipping'   => absint( $payment_settings['shipping'] ),
			'notify_url'    => add_query_arg( 'wpforms-listener', 'IPN', home_url( 'index.php' ) ),
			'return'        => $return_url,
			'rm'            => '1',
			'tax'           => 0,
			'upload'        => '1',
		);

		// Add cart items.
		if ( '_cart' === $transaction ) {

			// Product/service.
			$i = 1;
			foreach ( $items as $item ) {

				$item_amount = wpforms_sanitize_amount( $item['amount'] );

				if ( empty( $item['name'] ) ) {
					$item['name'] = sprintf(
						'Field %s',
						$item['id']
					);
				}

				if (
					! empty( $item['value_choice'] ) &&
					in_array( $item['type'], array( 'payment-multiple', 'payment-select', 'payment-checkbox' ), true )
				) {
					$item['value_choice'] = ( 'payment-checkbox' === $item['type'] ) ? str_replace( "\r\n", ', ', $item['value_choice'] ) : $item['value_choice'];
					$item_name            = $item['name'] . ' - ' . $item['value_choice'];
				} else {
					$item_name = $item['name'];
				}
				$paypal_args[ 'item_name_' . $i ] = stripslashes_deep( html_entity_decode( $item_name, ENT_COMPAT, 'UTF-8' ) );
				// Don't yet support quantities.
				//$paypal_args['quantity_' . $i ]  = $item['quantity'];
				$paypal_args[ 'amount_' . $i ] = $item_amount;
				$i ++;
			}
		} else {

			// Combine a donation name from all payment fields names.
			$item_names = array();

			foreach ( $items as $item ) {

				if ( empty( $item['name'] ) ) {
					$item['name'] = sprintf(
						'Field %s',
						$item['id']
					);
				}

				if (
					! empty( $item['value_choice'] ) &&
					in_array( $item['type'], array( 'payment-multiple', 'payment-select', 'payment-checkbox' ), true )
				) {
					$item['value_choice'] = ( 'payment-checkbox' === $item['type'] ) ? str_replace( "\r\n", ', ', $item['value_choice'] ) : $item['value_choice'];
					$item_name            = $item['name'] . ' - ' . $item['value_choice'];
				} else {
					$item_name = $item['name'];
				}

				$item_names[] = stripslashes_deep( html_entity_decode( $item_name, ENT_COMPAT, 'UTF-8' ) );
			}

			$paypal_args['item_name'] = implode( '; ', $item_names );
			$paypal_args['amount']    = $amount;
		}

		// Last change to filter args.
		/**
		 * PayPal Standard redirect arguments.
		 *
		 * @since 1.0.0
		 *
		 * @param array  $paypal_args PayPal arguments.
		 * @param array  $fields      Form fields.
		 * @param array  $form_data   Form data.
		 * @param string $entry_id    Entry ID.
		 */
		$paypal_args = apply_filters( 'wpforms_paypal_redirect_args', $paypal_args, $fields, $this->form_data, $entry_id ); // phpcs:ignore WPForms.PHP.ValidateHooks.InvalidHookName

		// Build query.
		$redirect .= http_build_query( $paypal_args );
		$redirect  = str_replace( '&amp;', '&', $redirect );

		// Redirect to PayPal.
		wp_redirect( $redirect );
		exit;
	}

	/**
	 * Log payment error.
	 *
	 * @since 1.6.0
	 *
	 * @param string       $title    Error title.
	 * @param array|string $messages Error messages.
	 * @param string       $level    Error level to add to 'payment' error level.
	 */
	private function log_errors( $title, $messages = [], $level = 'error' ) {

		wpforms_log(
			$title,
			$messages,
			[
				'type'    => [ 'payment', $level ],
				'form_id' => $this->form_data['id'],
			]
		);
	}

	/**
	 * Display form errors.
	 *
	 * @since 1.6.0
	 *
	 * @param array $errors Errors to display.
	 */
	private function display_errors( $errors ) {

		if ( ! $errors || ! is_array( $errors ) ) {
			return;
		}

		wpforms()->get( 'process' )->errors[ $this->form_data['id'] ]['footer'] = implode( '<br>', $errors );
	}

	/**
	 * Check if conditional logic check passes for the given settings.
	 *
	 * @since 1.4.0
	 *
	 * @param array $fields Form fields.
	 *
	 * @return bool
	 */
	private function is_conditional_logic_ok( $fields ) {

		// Check for conditional logic.
		if (
			empty( $this->form_data['payments'][ $this->slug ]['conditional_logic'] ) &&
			empty( $this->form_data['payments'][ $this->slug ]['conditional_type'] ) &&
			empty( $this->form_data['payments'][ $this->slug ]['conditionals'] )
		) {
			return true;
		}

		// All conditional logic checks passed, continue with processing.
		$process = wpforms_conditional_logic()->conditionals_process( $fields, $this->form_data, $this->form_data['payments'][ $this->slug ]['conditionals'] );

		if ( $this->form_data['payments'][ $this->slug ]['conditional_type'] === 'stop' ) {
			$process = ! $process;
		}

		return $process;
	}

	/**
	 * Process PayPal IPN.
	 *
	 * Adapted from EDD and the PHP PayPal IPN Class.
	 *
	 * @link https://github.com/easydigitaldownloads/Easy-Digital-Downloads/blob/master/includes/gateways/paypal-standard.php
	 * @link https://github.com/WadeShuler/PHP-PayPal-IPN/blob/master/src/IpnListener.php
	 *
	 * @since 1.0.0
	 */
	public function process_ipn() {

		// Verify the call back query and its method.
		if (
			! isset( $_GET['wpforms-listener'] ) ||
			'IPN' !== $_GET['wpforms-listener'] ||
			(
				isset( $_SERVER['REQUEST_METHOD'] ) &&
				'POST' !== $_SERVER['REQUEST_METHOD']
			)
		) {
			return;
		}

		// Set initial post data to empty string.
		$post_data = '';

		// Fallback just in case post_max_size is lower than needed.
		if ( ini_get( 'allow_url_fopen' ) ) {
			$post_data = file_get_contents( 'php://input' );
		} else {
			// If allow_url_fopen is not enabled, then make sure that post_max_size is large enough.
			ini_set( 'post_max_size', '12M' );
		}

		// Start the encoded data collection with notification command.
		$encoded_data = 'cmd=_notify-validate';

		// Verify there is a post_data.
		if ( $post_data || strlen( $post_data ) > 0 ) {
			// Append the data.
			$encoded_data .= ini_get( 'arg_separator.output' ) . $post_data;
		} else {
			// Check if POST is empty.
			if ( empty( $_POST ) ) {
				// Nothing to do.
				return;
			}

			// Loop through each POST.
			foreach ( $_POST as $key => $value ) {
				// Encode the value and append the data.
				$encoded_data .= ini_get( 'arg_separator.output' ) . "$key=" . rawurlencode( $value );
			}
		}

		// Convert collected post data to an array.
		parse_str( $encoded_data, $data );

		foreach ( $data as $key => $value ) {
			if ( false !== strpos( $key, 'amp;' ) ) {
				$new_key = str_replace( array( '&amp;', 'amp;' ), '&', $key );
				unset( $data[ $key ] );
				$data[ $new_key ] = $value;
			}
		}

		// Check if $post_data_array has been populated.
		if ( ! is_array( $data ) || empty( $data ) || empty( $data['invoice'] ) ) {
			return;
		}

		$error           = '';
		$invoice         = explode( '-', $data['invoice'], 2 );
		$payment_id      = ! empty( $invoice[1] ) ? absint( $invoice[1] ) : absint( $data['invoice'] );
		$payment         = wpforms()->entry->get( absint( $payment_id ) );
		$payment_status  = strtolower( $data['payment_status'] );
		$this->form_data = wpforms()->form->get(
			$payment->form_id,
			[
				'content_only' => true,
			]
		);

		// If payment or form doesn't exist, bail.
		if ( empty( $payment ) || empty( $this->form_data ) ) {
			return;
		}

		$payment_meta = json_decode( $payment->meta, true );

		$is_production_mode = ! empty( $payment_meta['payment_mode'] ) && $payment_meta['payment_mode'] === self::PRODUCTION;

		// Verify IPN with PayPal unless specifically disabled.
		$remote_post_args = array(
			'method'      => 'POST',
			'timeout'     => 45,
			'redirection' => 5,
			'httpversion' => '1.1',
			'blocking'    => true,
			'headers'     => array(
				'host'         => $is_production_mode ? 'www.paypal.com' : 'www.sandbox.paypal.com',
				'connection'   => 'close',
				'content-type' => 'application/x-www-form-urlencoded',
				'post'         => '/cgi-bin/webscr HTTP/1.1',
				'user-agent'   => 'WPForms IPN Verification',
			),
			'body'        => $data,
		);
		$remote_post_url  = $is_production_mode ? 'https://ipnpb.paypal.com/cgi-bin/webscr' : 'https://ipnpb.sandbox.paypal.com/cgi-bin/webscr';
		$remote_post      = wp_remote_post( $remote_post_url, $remote_post_args );

		if ( is_wp_error( $remote_post ) || wp_remote_retrieve_body( $remote_post ) !== 'VERIFIED' ) {
			wpforms_log(
				esc_html__( 'PayPal Standard IPN Error', 'wpforms-paypal-standard' ),
				$remote_post,
				array(
					'parent'  => $payment_id,
					'type'    => array( 'error', 'payment' ),
					'form_id' => $payment->form_id,
				)
			);

			return;
		}

		// Verify transaction type.
		if ( 'web_accept' !== $data['txn_type'] && 'cart' !== $data['txn_type'] ) {
			return;
		}

		$email = $this->get_email();
		// Verify payment recipient emails match.
		if ( empty( $email ) || strtolower( $data['business'] ) !== strtolower( trim( $email ) ) ) {
			$error = esc_html__( 'Payment failed: recipient emails do not match', 'wpforms-paypal-standard' );

		// Verify payment currency.
		} elseif ( empty( $payment_meta['payment_currency'] ) || strtolower( $data['mc_currency'] ) !== strtolower( $payment_meta['payment_currency'] ) ) {
			$error = esc_html__( 'Payment failed: currency formats do not match', 'wpforms-paypal-standard' );

		// Verify payment amounts.
		} elseif ( empty( $payment_meta['payment_total'] ) || number_format( (float) $data['mc_gross'] ) !== number_format( (float) $payment_meta['payment_total'] ) ) {
			$error = esc_html__( 'Payment failed: payment amounts do not match', 'wpforms-paypal-standard' );
		}

		// If there was an error, log and update the payment status.
		if ( ! empty( $error ) ) {
			$payment_meta['payment_note'] = $error;
			wpforms_log(
				esc_html__( 'PayPal Standard IPN Error', 'wpforms-paypal-standard' ),
				sprintf( '%s - IPN data: %s', $error, '<pre>' . print_r( $data, true ) . '</pre>' ),
				array(
					'parent'  => $payment_id,
					'type'    => array( 'error', 'payment' ),
					'form_id' => $payment->form_id,
				)
			);
			wpforms()->entry->update(
				$payment_id,
				array(
					'status' => 'failed',
					'meta'   => wp_json_encode( $payment_meta ),
				),
				'',
				'',
				array( 'cap' => false )
			);
			$this->update_payment_meta( $payment, $payment_meta );

			return;
		}

		// Completed payment.
		if ( $payment_status === 'completed' || ! $is_production_mode ) {

			$payment_meta['payment_transaction'] = $data['txn_id'];
			$payment_meta['payment_note']        = '';
			wpforms()->entry->update(
				$payment_id,
				array(
					'status' => 'completed',
					'meta'   => wp_json_encode( $payment_meta ),
				),
				'',
				'',
				array( 'cap' => false )
			);
			$this->update_payment_meta( $payment, $payment_meta );

			// Send notification emails if configured.
			wpforms()->process->entry_email( wpforms_decode( $payment->fields ), [], $this->unset_non_paypal_notifications(), $payment_id, $this->slug );

		// Refunded payment.
		} elseif ( 'refunded' === $payment_status ) {
			/* translators: %s - Paypal payment transaction ID. */
			$payment_meta['payment_note'] = sprintf( esc_html__( 'Payment refunded: PayPal refund transaction ID: %s', 'wpforms-paypal-standard' ), $data['txn_id'] );
			wpforms()->entry->update(
				$payment_id,
				array(
					'status' => 'refunded',
					'meta'   => wp_json_encode( $payment_meta ),
				),
				'',
				'',
				array( 'cap' => false )
			);
			$this->update_payment_meta( $payment, $payment_meta );

			// Pending payment.
		} elseif ( 'pending' === $payment_status && isset( $data['pending_reason'] ) ) {

			switch ( strtolower( $data['pending_reason'] ) ) {
				case 'echeck':
					$note = esc_html__( 'Payment made via eCheck and will clear automatically in 5-8 days', 'wpforms-paypal-standard' );
					break;
				case 'address':
					$note = esc_html__( 'Payment requires a confirmed customer address and must be accepted manually through PayPal', 'wpforms-paypal-standard' );
					break;
				case 'intl':
					$note = esc_html__( 'Payment must be accepted manually through PayPal due to international account regulations', 'wpforms-paypal-standard' );
					break;
				case 'multi-currency':
					$note = esc_html__( 'Payment received in non-shop currency and must be accepted manually through PayPal', 'wpforms-paypal-standard' );
					break;
				case 'paymentreview':
				case 'regulatory_review':
					$note = esc_html__( 'Payment is being reviewed by PayPal staff as high-risk or in possible violation of government regulations', 'wpforms-paypal-standard' );
					break;
				case 'unilateral':
					$note = esc_html__( 'Payment was sent to non-confirmed or non-registered email address.', 'wpforms-paypal-standard' );
					break;
				case 'upgrade':
					$note = esc_html__( 'PayPal account must be upgraded before this payment can be accepted', 'wpforms-paypal-standard' );
					break;
				case 'verify':
					$note = esc_html__( 'PayPal account is not verified. Verify account in order to accept this payment', 'wpforms-paypal-standard' );
					break;
				case 'other':
					$note = esc_html__( 'Payment is pending for unknown reasons. Contact PayPal support for assistance', 'wpforms-paypal-standard' );
					break;
				default:
					$note = esc_html( $data['pending_reason'] );
					break;
			}

			$payment_meta['payment_note'] = $note;
			wpforms()->entry->update(
				$payment_id,
				array(
					'status' => 'pending',
					'meta'   => wp_json_encode( $payment_meta ),
				),
				'',
				'',
				array( 'cap' => false )
			);
			$this->update_payment_meta( $payment, $payment_meta );
		}

		/**
		 * Completed PayPal Standard IPN call.
		 *
		 * @since 1.0.0
		 *
		 * @param array  $fields     Payment fields.
		 * @param array  $form_data  Form data.
		 * @param string $payment_id Payment ID.
		 * @param string $data       Payment data.
		 */
		do_action( 'wpforms_paypal_standard_process_complete', wpforms_decode( $payment->fields ), $this->form_data, $payment_id, $data );

		exit;
	}

	/**
	 * Unset non PayPal notifications before process.
	 *
	 * @since 1.4.0
	 *
	 * @return array
	 */
	private function unset_non_paypal_notifications() {

		if ( empty( $this->form_data['settings']['notifications'] ) ) {
			return $this->form_data;
		}

		foreach ( $this->form_data['settings']['notifications'] as $id => $notification ) {
			if ( empty( $notification[ $this->slug ] ) ) {
				unset( $this->form_data['settings']['notifications'][ $id ] );
			}
		}

		return $this->form_data;
	}

	/**
	 * Add checkbox to form notification settings.
	 *
	 * @since 1.4.0
	 *
	 * @param \WPForms_Builder_Panel_Settings $settings WPForms_Builder_Panel_Settings class instance.
	 * @param int                             $id       Subsection ID.
	 */
	public function notification_settings( $settings, $id ) {

		wpforms_panel_field(
			'toggle',
			'notifications',
			$this->slug,
			$settings->form_data,
			esc_html__( 'Enable for PayPal Standard completed payments', 'wpforms-paypal-standard' ),
			[
				'parent'      => 'settings',
				'class'       => empty( $settings->form_data['payments'][ $this->slug ]['enable'] ) ? 'wpforms-hidden' : '',
				'input_class' => 'wpforms-radio-group wpforms-radio-group-' . $id . '-notification-by-status wpforms-radio-group-item-paypal_standard wpforms-notification-by-status-alert',
				'subsection'  => $id,
				'tooltip'     => wp_kses(
					__( 'When enabled this notification will <em>only</em> be sent when a PayPal Standard payment has been successfully <strong>completed</strong>.', 'wpforms-paypal-standard' ),
					[
						'em'     => [],
						'strong' => [],
					]
				),
				'data'        => [
					'radio-group'    => $id . '-notification-by-status',
					'provider-title' => esc_html__( 'PayPal Standard completed payments', 'wpforms-paypal-standard' ),
				],
			]
		);
	}

	/**
	 * Logic that helps decide if we should send completed payments notifications.
	 *
	 * @since 1.4.0
	 *
	 * @param bool   $process         Whether to process or not.
	 * @param array  $fields          Form fields.
	 * @param array  $form_data       Form data.
	 * @param int    $notification_id Notification ID.
	 * @param string $context         The context of the current email process.
	 *
	 * @return bool
	 */
	public function process_email( $process, $fields, $form_data, $notification_id, $context ) {

		if ( ! $process ) {
			return false;
		}

		if ( empty( $form_data['payments'][ $this->slug ]['enable'] ) ) {
			return $process;
		}

		if ( empty( $form_data['settings']['notifications'][ $notification_id ][ $this->slug ] ) ) {
			return $process;
		}

		if ( ! $this->is_conditional_logic_ok( $fields ) ) {
			return false;
		}

		return $context === $this->slug;
	}

	/**
	 * Enqueue assets for the builder.
	 *
	 * @since 1.5.0
	 *
	 * @param string $view Current view.
	 */
	public function enqueues( $view ) {

		$min = wpforms_get_min_suffix();

		wp_enqueue_script(
			'wpforms-builder-paypal-standard',
			WPFORMS_PAYPAL_STANDARD_URL . "assets/js/admin-builder-paypal-standard{$min}.js",
			[ 'wpforms-builder' ],
			WPFORMS_PAYPAL_STANDARD_VERSION,
			true
		);

		wp_localize_script(
			'wpforms-builder-paypal-standard',
			'wpforms_paypal_standard',
			[
				'payment_fields' => wpforms_payment_fields(),
			]
		);
	}

	/**
	 * Add localized strings to be available in the form builder.
	 *
	 * @since 1.5.0
	 *
	 * @param array   $strings Form builder JS strings.
	 * @param WP_Post $form    Current form.
	 *
	 * @return array
	 */
	public function javascript_strings( $strings, $form ) {

		$strings['paypal_standard_required_email'] = sprintf(
			wp_kses(
				'<p>%s</p><p>%s</p>',
				[
					'p'      => [],
					'strong' => [],
				]
			),
			__( 'PayPal Email Address must be filled in when using the PayPal payments.', 'wpforms-paypal-standard' ),
			__( 'To proceed, please go to <strong>Payments » Paypal Standard</strong> and fill in <strong>PayPal Email Address</strong>.', 'wpforms-paypal-standard' )
		);

		$strings['paypal_standard_required_payment_field'] = sprintf(
			wp_kses(
				'<p>%s</p><p>%s</p>',
				[
					'p'      => [],
					'strong' => [],
				]
			),
			__( 'PayPal Standard payments are enabled but not effective.', 'wpforms-paypal-standard' ),
			__( 'At least one <strong>Payment Field</strong> should be added to the form.', 'wpforms-paypal-standard' )
		);

		return $strings;
	}

	/**
	 * Insert payment data into entry_meta table.
	 *
	 * @since 1.5.0
	 *
	 * @param object $payment      Payment entry.
	 * @param array  $payment_meta Payment data.
	 *
	 * @return void
	 */
	private function update_payment_meta( $payment, $payment_meta ) {

		$entry_handler = wpforms()->get( 'entry' );

		if ( ! $entry_handler ) {
			return;
		}

		$current_payment_meta = json_decode( $payment->meta, true );
		$added_payment_meta   = array_diff( $payment_meta, $current_payment_meta );

		$entry_handler->insert_payment_meta( $payment->entry_id, $added_payment_meta );
	}

	/**
	 * Determine if production mode selected.
	 *
	 * @since 1.5.0
	 *
	 * @return bool
	 */
	private function is_production_mode() {

		return isset( $this->form_data['payments'][ $this->slug ]['mode'] ) && $this->form_data['payments'][ $this->slug ]['mode'] === self::PRODUCTION;
	}

	/**
	 * Get email address value.
	 *
	 * @since 1.5.0
	 *
	 * @return string
	 */
	private function get_email() {

		$settings = $this->form_data['payments'][ $this->slug ];

		if ( isset( $settings['production_email'], $settings['sandbox_email'] ) ) {
			return $this->is_production_mode() ? $settings['production_email'] : $settings['sandbox_email'];
		}

		// Backward compatibility for forms created on addon's version < 1.5.0.
		return isset( $settings['email'] ) ? $settings['email'] : '';
	}
}

new WPForms_Paypal_Standard();
